package cn.com.taiji.web.service.impl;

import cn.com.taiji.cql.model.GeoDataSource;
import cn.com.taiji.cql.service.IECqlService;
import cn.com.taiji.web.config.GeoConfig;
import cn.com.taiji.web.service.IGeoDataSourceService;
import com.alibaba.fastjson.JSONObject;
import lombok.extern.slf4j.Slf4j;
import org.geotools.data.jdbc.datasource.DBCPDataSourceFactory;
import org.geotools.data.jdbc.datasource.DataSourceFinder;
import org.geotools.jdbc.JDBCDataStoreFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.sql.DataSource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@Slf4j
@Service
public class GeoDataSourceServiceImpl implements IGeoDataSourceService {

    @Autowired
    private GeoConfig geoConfig;

    @Autowired
    private IECqlService ecqlService;

    @Override
    public GeoDataSource layerCodeGeoDataSource(String layerCode) throws Exception{


        //1先根据图层代码获取对应的数据源
        Map<String, Object> mapParams = new HashMap<String, Object>();
        mapParams.put(JDBCDataStoreFactory.DATABASE.key, geoConfig.getDsBase());
        mapParams.put(JDBCDataStoreFactory.DBTYPE.key, geoConfig.getDbType());

        mapParams.put(JDBCDataStoreFactory.HOST.key, geoConfig.getDsIp());
        mapParams.put(JDBCDataStoreFactory.PORT.key, geoConfig.getDsPort());
        mapParams.put(JDBCDataStoreFactory.USER.key, geoConfig.getDsUsername());
        mapParams.put(JDBCDataStoreFactory.PASSWD.key, geoConfig.getDsPassword());
        mapParams.put("tableName", "cql_data_source");

        String layerCodeCql = "layer_code='"+layerCode+"'";

        Map<String, Object> sortParams = new HashMap<String, Object>();

        List<Map<String, Object>> dataSourceList = ecqlService.cqlQuery(mapParams,layerCodeCql,sortParams);

        if(dataSourceList.size() >0){

            GeoDataSource gds = JSONObject.parseObject(JSONObject.toJSONString(dataSourceList.get(0)), GeoDataSource.class);

            return gds;

        }

        return null;



    }

    public DataSource defaultDataSource() throws Exception{

        //构建mysql数据库连接数据源--DataSource
        Map<String, Object> mapParams = new HashMap<String, Object>();
        mapParams.put(DBCPDataSourceFactory.DSTYPE.key, "DBCP");
        mapParams.put(DBCPDataSourceFactory.DRIVERCLASS.key, "com.mysql.cj.jdbc.Driver");
        mapParams.put(DBCPDataSourceFactory.JDBC_URL.key, "jdbc:mysql://"+geoConfig.getDsIp()+":"+geoConfig.getDsPort()+"/"+geoConfig.getDsBase()+"?useUnicode=true&characterEncoding=UTF-8&autoReconnect=true&useSSL=false&zeroDateTimeBehavior=convertToNull&serverTimezone=Asia/Shanghai");
        mapParams.put(DBCPDataSourceFactory.USERNAME.key, geoConfig.getDsUsername());
        mapParams.put(DBCPDataSourceFactory.PASSWORD.key, geoConfig.getDsPassword());
        mapParams.put(DBCPDataSourceFactory.MAXACTIVE.key, Integer.valueOf(10));
        mapParams.put(DBCPDataSourceFactory.MAXIDLE.key, Integer.valueOf(0));

        return DataSourceFinder.getDataSource(mapParams);

    }
}
